//-------------------DEPTHSHARPENING-------------------//
// JawZ: Author and developer of this file             //
// Prod80: Initial depth based Sharpening code         //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//

float4 msGaussian(float2 inCoord, float2 dir)
{
  float2 pixelsize    = ScreenSize.y * dir;
         pixelsize.y *= ScreenSize.z;

  float Depth    = TextureDepth.Sample(Sampler0, inCoord.xy ).x;
  float linDepth = linearDepth( Depth, 0.5f, fFarDepth );

  float SigmaSum     = 0.0f;
  float sampleOffset = 1.0f;

/// Gaussian
      float fBlurSigma1 = max( fBlurSigma * ( 1.0f - linDepth ), 0.3f );
    float3 Sigma;
      Sigma.x = 1.0f / ( sqrt( 2.0f * PI ) * fBlurSigma1 );
      Sigma.y = exp( -0.5f / ( fBlurSigma1 * fBlurSigma1 ));
      Sigma.z = Sigma.y * Sigma.y;

/// Center weight
    float4 color = TextureColor.Sample(Sampler1, inCoord.xy);
    color  *= Sigma.x;
    SigmaSum += Sigma.x;
    Sigma.xy *= Sigma.yz;

    for(int i = 0; i < 7; ++i)
    {
      color     += TextureColor.Sample(Sampler1, inCoord.xy + pixelsize * sampleOffset) * Sigma.x;
      color     += TextureColor.Sample(Sampler1, inCoord.xy - pixelsize * sampleOffset) * Sigma.x;
      SigmaSum    += ( 2.0f * Sigma.x );
      sampleOffset = sampleOffset + 1.0f;
      Sigma.xy    *= Sigma.yz;
    }

  color.xyz /= SigmaSum;
  color.w    = 1.0f;
  return color;
}

float4 msEdges(float4 inColor, float2 inCoord)
{
  float4 orig    = TextureOriginal.Sample(Sampler1, inCoord.xy);
  float4 blurred = inColor;

/// Find edges
    orig.xyz           = saturate( orig.xyz );
    blurred.xyz        = saturate( blurred.xyz );
    float3 Edges       = max( saturate( orig.xyz - blurred.xyz ) - fThreshold, 0.0f );
    float3 invBlur     = saturate( 1.0f - blurred.xyz );
    float3 originvBlur = saturate( orig.xyz + invBlur.xyz );
    float3 invOrigBlur = max( saturate( 1.0f - originvBlur.xyz ) - fThreshold, 0.0f );

    float3 edges = max(( saturate( fSharpening * Edges.xyz )) - ( saturate( fSharpening * invOrigBlur.xyz )), 0.0f );

  return float4(edges, 1.0);
}

float4 msSharpen(float2 inCoord, float2 dir)
{
/// Smooth out edges (reduce aliasing) - expensive, likely

    float2 pixelsize    = ScreenSize.y * dir;
           pixelsize.y *= ScreenSize.z;

    float SigmaSum     = 0.0f;
    float sampleOffset = 1.0f;

/// Gaussian
    float BlurSigma = fSmooth;
    float3 Sigma;
    Sigma.x         = 1.0f / ( sqrt( 2.0f * PI ) * BlurSigma );
    Sigma.y         = exp( -0.5f / ( BlurSigma * BlurSigma ));
    Sigma.z         = Sigma.y * Sigma.y;

/// Center weight
    float4 edges = TextureColor.Sample(Sampler1, inCoord.xy);
    edges    *= Sigma.x;
    SigmaSum += Sigma.x;
    Sigma.xy *= Sigma.yz;

    for(int i = 0; i < 5; ++i)
    {
      edges       += TextureColor.Sample(Sampler1, inCoord.xy + sampleOffset * pixelsize) * Sigma.x;
      edges       += TextureColor.Sample(Sampler1, inCoord.xy - sampleOffset * pixelsize) * Sigma.x;
      SigmaSum    += ( 2.0f * Sigma.x );
      sampleOffset = sampleOffset + 1.0f;
      Sigma.xy    *= Sigma.yz;
    }

  return float4( edges.xyz / SigmaSum, 1.0);
}
